@@ -57,4 +57,9 @@
 #define W_ONE       W_CONST(1)
 #define W_FFFF      W_CONST(~0)
 
+#define BITS_PER_WORD (8 * sizeof(WTYPE))
+#define MAXBIT        (BITS_PER_WORD-1)
+#define NWORDS(bits)  ( ((bits)+BITS_PER_WORD-1) / BITS_PER_WORD )
+#define NBYTES(bits)  ( ((bits)+8-1) / 8 )
+
 #endif
